#include <inttypes.h>      // common scalar types
#include <avr/io.h>        // for device register definitions
#include <avr/interrupt.h> // for interrupt enable
#include "main.h"         // look for definitions here
#include "cdc.h"


static struct
{	// all the information for cdc
	volatile enum
	{
		eIdle = 0,
		ePreparing = 1,
		eStopped = 2,
		ePaused = 3,
		ePlaying = 4,
		eSpinup = 5,
		eFF = 6,
		eRW = 7,
	} state;
	uint8_t uDisk; // current disc
	uint8_t uTrackL; // current track low byte
	uint8_t uTrackH; // current track hi byte
} gCdc; 


void cdc_init (void)
{
	gCdc.state = eIdle;
	gCdc.uDisk = def_disk;
	gCdc.uTrackL = def_track_l;
	gCdc.uTrackH = def_track_h;
}

void cdc_process_command (void)
{
	uint8_t fetched;
	fetched = MbusRxReadByte();
	if (fetched == 0x01) 			//processing only headunit commands
	{

		fetched = MbusRxReadByte();
		switch (fetched)
		{
			case 0x08:	//ping
				cdc_ping_ok();
				break;

//			case 0x09: //wake up
//				cdc_changing(gCdc.uDisk, 0, 0, 0x0C);
//				cdc_some_status();
//				cdc_disk_info(gCdc.uDisk, 0, 0, 0, 0, 0, 0);
//				cdc_play_state(0, 4, 0, 0, 0, 0, 0, 0);
//
//				break;
			case 0x01:

				fetched = MbusRxReadByte();
				switch (fetched)
				{
					case 0x01:		//set play state
						{
							uint8_t flags2, flags1;
							flags2 = MbusRxReadByte();
							flags1 = MbusRxReadByte();
							if (flags2 & 4) gCdc.state = eStopped;
							if (flags1 & 2) gCdc.state = ePaused;
							if (flags1 & 1) gCdc.state = ePlaying;
							if (flags1 & 4) gCdc.state = eFF;
							if (flags1 & 8) gCdc.state = eRW;
						}
						if (gCdc.state == ePlaying) 
						{
							cdc_disk_info(gCdc.uDisk, tracks_h, tracks_l, min_h, min_l, sec_h, sec_l);
							m32_send_command(m32_PLAY);
						}
						if (gCdc.state == eStopped) 
						{
							m32_send_command(m32_STOP);
						}
						cdc_play_state(0, 1, 0, 0, 0, 0, 0, 0);
						break;

					case 0x03:		//set disk+track
					
						fetched = MbusRxReadByte();
						if (fetched != 0) 
						{
							cdc_changing(gCdc.uDisk, 0, 0, 1, 0, 0, 1);
							gCdc.uDisk = fetched;
							gCdc.uTrackH = 0;
							gCdc.uTrackL = 1;
							cdc_changing(gCdc.uDisk, 0, 0, 0, 0, 0, 0x0C);
							gCdc.state = eSpinup;
							cdc_play_state(0, 5, 0, 0, 0, 0, 0, 0);
							cdc_disk_info(gCdc.uDisk, tracks_h, tracks_l, min_h, min_l, sec_h, sec_l);
						}
						else
						{
							uint8_t track_old, track_new;

							track_old = gCdc.uTrackH;
							track_old = track_old << 4;
							track_old += gCdc.uTrackL;


							gCdc.uTrackH = MbusRxReadByte();
							gCdc.uTrackL = MbusRxReadByte();

							track_new = gCdc.uTrackH;
							track_new = track_new << 4;
							track_new += gCdc.uTrackL;

							if (track_old < track_new)
							{
								if ((track_new == 0x99) && (track_old == 0x01))
								{
									m32_send_command(m32_PREV);
								}
								else
								{
									m32_send_command(m32_NEXT);
								}
							}
							else if(track_old > track_new)
							{
								if ((track_new == 0x01) && (track_old == 0x99))
								{
									m32_send_command(m32_NEXT);
								}
								else
								{
									m32_send_command(m32_PREV);
								}
							}

							cdc_changing(gCdc.uDisk, gCdc.uTrackH, gCdc.uTrackL, 0, 0, 0, 1);
							gCdc.state = eSpinup;
							cdc_play_state(0, 5, 0, 0, 0, 0, 0, 0);
						}
						
						gCdc.state = ePlaying;
						cdc_play_state(0, 5, 0, 0, 0, 0, 0, 0);
						break;

	//							case 0x04:		//set programm mode
					
	//								break;

					default:
						cdc_disk_info(gCdc.uDisk, tracks_h, tracks_l, min_h, min_l, sec_h, sec_l);
						cdc_play_state(0, 5, 0, 0, 0, 1, 0, 0);
				}
			
				break;
	
			default:
				cdc_ack_wait();
		}


	}


	while (fetched != 0xF0)
	{
		fetched = MbusRxReadByte();
	}


}

void cdc_update_status(void)
{
	if(gCdc.state == ePlaying) cdc_play_state(0, 5, 0, 0, 0, 0, 0, 0);
}

void cdc_ping_ok(void)
{
	MbusTxWriteByte(0x09);	
	MbusTxWriteByte(0x08);	//ping ok
	MbusTxWriteByte(0x02);
	MbusTxWriteByte(0xF0);
}

void cdc_ack_wait(void)
{
	MbusTxWriteByte(0x09);
	MbusTxWriteByte(0x0F);
	MbusTxWriteByte(0x00);
	MbusTxWriteByte(0x00);
	MbusTxWriteByte(0x00);
	MbusTxWriteByte(0x00);
	MbusTxWriteByte(0x00);
	MbusTxWriteByte(0x07);
	MbusTxWriteByte(0xF0);
}

void cdc_changing(uint8_t Disk, uint8_t TrackH, uint8_t TrackL, uint8_t Flags3, uint8_t Flags2, uint8_t Flags1, uint8_t Flags0)
{
	uint8_t crc;

	crc = 0;
	MbusTxWriteByte(0x09);
	crc ^= 0x09;	
	MbusTxWriteByte(0x0B);	
	crc ^= 0x0B;	
	MbusTxWriteByte(0x09);	
	crc ^= 0x09;	
	MbusTxWriteByte(Disk);	
	crc ^= Disk;	
	MbusTxWriteByte(TrackH);	
	crc ^= TrackH;	
	MbusTxWriteByte(TrackL);	
	crc ^= TrackL;	
	MbusTxWriteByte(Flags3);	
	crc ^= Flags3;	
	MbusTxWriteByte(Flags2);	
	crc ^= Flags2;	
	MbusTxWriteByte(Flags1);	
	crc ^= Flags1;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(Flags0);	
	crc ^= Flags0;	
	crc += 1;	
	MbusTxWriteByte(crc);	
	MbusTxWriteByte(0xF0);	
}

void cdc_disk_info(uint8_t Disk, uint8_t TracksH, uint8_t TracksL, uint8_t MinH, uint8_t MinL, uint8_t SecH, uint8_t SecL)
{
	uint8_t crc;

	crc = 0;
	MbusTxWriteByte(0x09);
	crc ^= 0x09;	
	MbusTxWriteByte(0x0C);	
	crc ^= 0x0C;	
	MbusTxWriteByte(Disk);	
	crc ^= Disk;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x01);	
	crc ^= 0x01;	
	MbusTxWriteByte(TracksH);	
	crc ^= TracksH;	
	MbusTxWriteByte(TracksL);	
	crc ^= TracksL;	
	MbusTxWriteByte(MinH);	
	crc ^= MinH;	
	MbusTxWriteByte(MinL);	
	crc ^= MinL;	
	MbusTxWriteByte(SecH);	
	crc ^= SecH;	
	MbusTxWriteByte(SecL);	
	crc ^= SecL;	
	MbusTxWriteByte(0x0F);	
	crc ^= 0x0F;	
	crc += 1;	
	MbusTxWriteByte(crc);	
	MbusTxWriteByte(0xF0);	
}

void cdc_play_state(uint8_t IndexH, uint8_t IndexL, uint8_t MinH, uint8_t MinL, uint8_t SecH, uint8_t SecL, uint8_t Flags3, uint8_t Flags2)
{
	uint8_t Flags1;
	uint8_t crc;

	switch (gCdc.state)
	{
		case eStopped:
			Flags1 = 8;
			break;
		case ePaused:
			Flags1 = 2;
			break;
		case ePlaying:
		case eSpinup:
		case eFF:
		case eRW:
			Flags1 = 1;
			break;
		case eIdle:
			Flags1 = 0x0C;
			break;
		default:
			Flags1 = 0;
	}


	crc = 0;	
	MbusTxWriteByte(0x09);	
	crc ^= 0x09;	
	MbusTxWriteByte(0x09);	
	crc ^= 0x09;	
	MbusTxWriteByte(gCdc.state);	
	crc ^= gCdc.state;	
	MbusTxWriteByte(gCdc.uTrackH);	
	crc ^= gCdc.uTrackH;	
	MbusTxWriteByte(gCdc.uTrackL);	
	crc ^= gCdc.uTrackL;	
	MbusTxWriteByte(IndexH);	
	crc ^= IndexH;	
	MbusTxWriteByte(IndexL);	
	crc ^= IndexL;	
	MbusTxWriteByte(MinH);	
	crc ^= MinH;	
	MbusTxWriteByte(MinL);	
	crc ^= MinL;	
	MbusTxWriteByte(SecH);	
	crc ^= SecH;	
	MbusTxWriteByte(SecL);	
	crc ^= SecL;	
	MbusTxWriteByte(Flags3);	
	crc ^= Flags3;	
	MbusTxWriteByte(Flags2);	
	crc ^= Flags2;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(Flags1);	
	crc ^= Flags1;	
	crc += 1;	
	MbusTxWriteByte(crc);	
	MbusTxWriteByte(0xF0);	
}

void cdc_some_status(void) //may be inserted disks
{
	uint8_t crc;

	crc = 0;
	MbusTxWriteByte(0x09);
	crc ^= 0x09;	
	MbusTxWriteByte(0x0D);	
	crc ^= 0x0D;	
	MbusTxWriteByte(0x01);	
	crc ^= 0x01;	
	MbusTxWriteByte(0x02);	
	crc ^= 0x02;	
	MbusTxWriteByte(0x03);	
	crc ^= 0x03;	
	MbusTxWriteByte(0x04);	
	crc ^= 0x04;	
	MbusTxWriteByte(0x05);	
	crc ^= 0x05;	
	MbusTxWriteByte(0x06);	
	crc ^= 0x06;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	MbusTxWriteByte(0x0F);	
	crc ^= 0x0F;	
	MbusTxWriteByte(0x0C);	
	crc ^= 0x0C;	
	MbusTxWriteByte(0x00);	
	crc ^= 0x00;	
	crc += 1;	
	MbusTxWriteByte(crc);	
	MbusTxWriteByte(0xF0);	
}
