#ifndef _MAIN_H
#define _MAIN_H

// a place for platform dependent indirect definitions
#if defined(__AVR_ATmega8__) // trying the Mega8 now
#define PIN_MBUS_OUT   PORTB1 // MBus output pin (active = pull low)

#define PIN_POWER_ON   PIND2
#define PIN_POWER_RELAY   PORTD3

#else
#error "Code is not prepared for that MCU!"
#endif

#define SYSCLK 7372800L	// system clock in Hertz
#define BAUDRATE 115200UL // default UART baudrate

#define PRESCALE_256 _BV(CS02) // prescaler for normal operation
#define PRESCALE_1   _BV(CS00) // minimum prescaler to force interrupt

// in timer ticks (SYSCLK / prescaler = 34.722 us)
#define DEFAULT_ZERO_TIME 20 // low pulse 0.6 ms: "0" bit
#define DEFAULT_ONE_TIME  54 // low pulse 1.9 ms: "1" bit
#define DEFAULT_BIT_TIME  89 // time for a full bit cycle
#define DEFAULT_TOLERANCE 15 // the allowed "jitter" on reception
#define DEFAULT_MIN_PAUSE 10 // timeout for packet completion
#define DEFAULT_SPACE     50 // pause before sending new packet

#define MIN_ZERO_TIME  (DEFAULT_ZERO_TIME - DEFAULT_TOLERANCE)
#define MAX_ZERO_TIME  (DEFAULT_ZERO_TIME + DEFAULT_TOLERANCE)
#define MIN_ONE_TIME   (DEFAULT_ONE_TIME - DEFAULT_TOLERANCE)
#define MAX_ONE_TIME   (DEFAULT_ONE_TIME + DEFAULT_TOLERANCE)
#define BIT_TIMEOUT    (DEFAULT_BIT_TIME + DEFAULT_MIN_PAUSE)
#define SEND_ZERO_TIME (DEFAULT_ZERO_TIME)
#define SEND_ONE_TIME  (DEFAULT_ONE_TIME)
#define SEND_BIT_TIME  (DEFAULT_BIT_TIME)
#define SEND_SPACE     (DEFAULT_SPACE)



#define m32_addr_r 0b00000011
#define m32_addr_w 0b00000010

#define m32_IDLE 0
#define m32_PLAY 1
#define m32_PLAYING 2
#define m32_STOP 3
#define m32_STOPPED 4
#define m32_NEXT 5
#define m32_PREV 6
#define m32_CHANGING 7
#define m32_SAVE 8


// prototypes
int main (void); // here we go...
void init_io (void); // helper function for main(): setup timers and pins
void init_eeprom (void); // a convenience feature to populate the timings in eeprom with reasonable defaults
char int2hex(uint8_t n); // utility function: convert a number to a hex char
uint8_t hex2int(char c); // utility function: convert a hex char to a number

uint8_t MbusRxReadByte(void);
void MbusRxWriteByte(uint8_t data);
uint8_t MbusTxReadByte(void);
void MbusTxWriteByte(uint8_t data);
uint8_t MbusDataInRxBuffer(void);
uint8_t MbusSearchInRxBuffer(uint8_t key);
uint8_t MbusDataInTxBuffer(void);
uint8_t MbusSearchInTxBuffer(uint8_t key);

void m32_send_command(uint8_t command);
uint8_t m32_read_status(void);

// interrupt handlers
ISR(TIMER0_OVF_vect); // timer 0 overflow, used for sending
ISR(TIMER1_CAPT_vect); // edge detection interrupt, the heart of receiving
ISR(TIMER1_COMPA_vect); // timeout interrupt, this terminates a received packet
ISR(TIMER2_OVF_vect); //system timer
ISR(TWI_vect); //i2c bus
#endif
